%module parted

%{
#include <parted/parted.h>
%}

%section "Original copyright notice"

/*
    libparted - a library for manipulating disk partitions
    Copyright (C) 1999, 2000 Free Software Foundation, Inc.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

%section "parted.h"

#include <stdlib.h>

%name(init)
extern int ped_init ();
%name(done)
extern void ped_done ();
%name(get_version)
extern const char* ped_get_version ();


%section "device.h"

/* FIXME: make this work */
/* typedef long long PedSector; */

typedef enum {
	PED_DEVICE_UNKNOWN	= 0,
	PED_DEVICE_SCSI		= 1,
	PED_DEVICE_IDE		= 2,
	PED_DEVICE_DAC960	= 3,
	PED_DEVICE_CPQARRAY	= 4,
	PED_DEVICE_FILE		= 5,
	PED_DEVICE_ATARAID	= 6
} PedDeviceType;

/* A hard disk device - eg. /dev/hda, not /dev/hda3 */
%rename PedDevice Device;
typedef struct {
/* 	PedDevice*	next; */

	char*		model;			/* description of hardware */
	char*		path;			/* device /dev entry */

	PedDeviceType	type;			/* SCSI, IDE, etc. */
	int		sector_size;
	PedSector	length;

/* 	int		open_count; */
	int		read_only;
/* 	int		external_mode; */
/* 	int		dirty; */
/* 	int		boot_dirty; */
/* 	int		fd; */

/* x86-specific: TODO hide this in arch_specific.  But argghh!  The disk
 * label stuff needs it, not to mention front-ends :-( */

/* 	int		heads, sectors, cylinders; */
/* 	int		geom_known; */
/* 	int		geom_already_guessed; */
/* 	short		host, did; */

} PedDevice;

%name(device_probe_all)
extern void ped_device_probe_all ();

/* FIXME: implement devices_get instead */
%{
PedDevice* ped_device_get_first () {
  return ped_device_get_next (NULL);
}
 %}
/* this must be declared after PedDevice itself */
%name(device_get_first)
extern PedDevice* ped_device_get_first ();


%name(device_get)
extern PedDevice* ped_device_get (const char* name);

%addmethods PedDevice {
  %name(get_next)
     PedDevice* ped_device_get_next ();
  %name(open)
     int ped_device_open ();
  %name(close)
     int ped_device_close ();
  %name(begin_external_access)
     int ped_device_begin_external_access ();
  %name(end_external_access)
     int ped_device_end_external_access ();
  %name(read)
     int ped_device_read (void* buffer,
			  PedSector start, PedSector count);
  %name(write)
     int ped_device_write (const void* buffer,
			   PedSector start, PedSector count);
  %name(sync)
     int ped_device_sync ();
  %name(check)
     PedSector ped_device_check (void* buffer,
				 PedSector start, PedSector count);
  %name(get_part_path)
     char* ped_device_get_part_path (int num);
}


%section "disk.h"

%rename PedGeometry Geometry;
typedef struct {
	PedDisk*		disk;
	PedSector		start;
	PedSector		length;
	PedSector		end;
} PedGeometry;

typedef enum {
	PED_PARTITION_PRIMARY		= 0x00,
	PED_PARTITION_LOGICAL		= 0x01,
	PED_PARTITION_EXTENDED		= 0x02,
	PED_PARTITION_FREESPACE		= 0x04,
	PED_PARTITION_METADATA		= 0x08
} PedPartitionType;

typedef enum {
	PED_PARTITION_BOOT=1,
	PED_PARTITION_ROOT=2,
	PED_PARTITION_SWAP=3,
	PED_PARTITION_HIDDEN=4,
	PED_PARTITION_RAID=5,
	PED_PARTITION_LVM=6,
	PED_PARTITION_LBA=7
} PedPartitionFlag;
#define PED_PARTITION_FIRST_FLAG	PED_PARTITION_BOOT
#define PED_PARTITION_LAST_FLAG		PED_PARTITION_LBA

typedef enum {
	PED_DISK_TYPE_EXTENDED=1,
	PED_DISK_TYPE_PARTITION_NAME=2
} PedDiskTypeFeature;
#define PED_DISK_TYPE_FIRST_FEATURE	PED_DISK_TYPE_EXTENDED
#define PED_DISK_TYPE_LAST_FEATURE	PED_DISK_TYPE_PARTITION_NAME

%rename PedPartition Partition;
typedef struct {
	PedPartition*		prev;
	PedPartition*		next;

	PedGeometry		geom;
	int			num;

	PedPartitionType	type;
	const PedFileSystemType* fs_type;
	PedPartition*		part_list; /* for extended partitions */
} PedPartition;

%rename PedDisk Disk;
typedef struct {
	PedDevice*		dev;
	PedDiskType*		type;
	PedPartition*		part_list;

	void*			disk_specific;

/* office use only ;-) */
	int			update_mode;
} PedDisk;

%rename PedDiskType DiskType;
typedef struct {
	const char*		name;
	PedDiskTypeFeature	features;
} PedDiskType;

%addmethods PedDiskType {
  %name(register)
     void ped_register_disk_type ();
  %name(unregister)
     void ped_unregister_disk_type ();
  %name(get_next)
     PedDiskType* ped_disk_type_get_next ();
  %name(check_feature)
     int ped_disk_type_check_feature (PedDiskTypeFeature feature);
}

%name(disk_type_get) PedDiskType* ped_disk_type_get (const char* name);

%addmethods PedDevice {
  %name(disk_probe)
     PedDiskType* ped_disk_probe ();
  %name(disk_open)
     PedDisk* ped_disk_open ();
  %name(disk_create)
     PedDisk* ped_disk_create (PedDiskType* type);
}

%addmethods PedDisk {
  %name(clobber)
     int ped_disk_clobber ();
  %name(close)
     int ped_disk_close ();
  %name(read)
     int ped_disk_read ();
  %name(write)
     int ped_disk_write ();
  %name(is_busy)
     int ped_disk_is_busy ();

  %name(get_primary_partition_count)
     int ped_disk_get_primary_partition_count ();
  %name(get_last_partition_num)
     int ped_disk_get_last_partition_num ();
  %name(get_max_primary_partition_count)
     int ped_disk_get_max_primary_partition_count ();
}

%addmethods PedDevice {
  %name(disk_alloc)
     PedDisk* ped_disk_alloc (PedDiskType* type);
}
%addmethods PedDisk {
  %name(free)
     void ped_disk_free ();
}

%addmethods PedGeometry {
  %name(init)
     int ped_geometry_init (const PedDisk* disk,
			    PedSector start, 
			    PedSector length);
}

%addmethods PedDisk {
  %name(geometry_new)
     PedGeometry* ped_geometry_new (PedSector start,
				    PedSector length);
}


%addmethods PedGeometry {
  %name(duplicate)
     PedGeometry* ped_geometry_duplicate ();
  %name(intersect)
     PedGeometry* ped_geometry_intersect (const PedGeometry* b);
  %name(destroy)
     void ped_geometry_destroy ();
  %name(set)
     int ped_geometry_set (PedSector start,
			   PedSector length);
  %name(set_start)
     int ped_geometry_set_start (PedSector start);
  %name(set_end)
     int ped_geometry_set_end (PedSector end);

  /* NOTE: changed name */
  %name(overlaps)
     int ped_geometry_test_overlap (const PedGeometry* b);
  /* NOTE: changed name */
  %name(contains)
     int ped_geometry_test_inside (const PedGeometry* b);
  /* NOTE: changed name */
  %name(equals)
     int ped_geometry_test_equal (const PedGeometry* b);
/* FIXME: semantics ?
  extern int ped_geometry_test_sector_inside (const PedGeometry* geom,
					      PedSector sect);
*/

  %name(read)
     int ped_geometry_read (void* buffer,
			    PedSector offset, PedSector count);
  %name(write)
     int ped_geometry_write (const void* buffer,
			     PedSector offset, PedSector count);
  %name(check)
     PedSector ped_geometry_check (void* buffer,
				   PedSector buffer_size, PedSector offset,
				   PedSector granularity, PedSector count);
  %name(sync)
     int ped_geometry_sync ();

/* returns -1 if "sector" is not within dest's space. */
  %name(map)
     PedSector ped_geometry_map (const PedGeometry* src,
				 PedSector sector);
}

extern PedPartition* ped_partition_alloc (const PedDisk* disk,
					  PedPartitionType type,
					  const PedFileSystemType* fs_type,
					  PedSector start,
					  PedSector end);
%addmethods PedDisk {
  %name(partition_alloc)
     PedPartition* ped_partition_alloc (PedPartitionType type,
					const PedFileSystemType* fs_type,
					PedSector start,
					PedSector end);
  %name(partition_new)
     PedPartition* ped_partition_new (PedPartitionType type,
				      const PedFileSystemType* fs_type,
				      PedSector start,
				      PedSector end);
}

%addmethods PedPartition {
  %name(destroy)
     void ped_partition_destroy (PedPartition* part);
  %name(is_active)
     int ped_partition_is_active ();
  %name(set_flag)
     int ped_partition_set_flag (PedPartitionFlag flag,
				 int state);
  %name(get_flag)
     int ped_partition_get_flag (PedPartitionFlag flag);
  %name(is_flag_available)
     int ped_partition_is_flag_available (PedPartitionFlag flag);
  %name(set_system)
     int ped_partition_set_system (const PedFileSystemType* fs_type);
  %name(set_name)
     int ped_partition_set_name (const char* name);
  %name(get_name)
     const char* ped_partition_get_name ();
  %name(is_busy)
     int ped_partition_is_busy ();
}

%name(partition_type_get_name) const char* ped_partition_type_get_name (PedPartitionType part_type);
%name(partition_flag_get_name) const char* ped_partition_flag_get_name (PedPartitionFlag flag);
%name(partition_flag_get_by_name) PedPartitionFlag ped_partition_flag_get_by_name (const char* name);
%name(partition_flag_next) PedPartitionFlag ped_partition_flag_next (PedPartitionFlag flag);

%addmethods PedDisk {
  %name(add_partition)
     int ped_disk_add_partition (PedPartition* part,
				 const PedConstraint* constraint);
  %name(remove_partition)
     int ped_disk_remove_partition (PedPartition* part);
  %name(delete_partition)
     int ped_disk_delete_partition (PedPartition* part);
  %name(delete_all)
     int ped_disk_delete_all ();
}

/* FIXME: why is the PedDisk necessary ?
extern int ped_disk_set_partition_geom (PedDisk* disk, PedPartition* part,
					const PedConstraint* constraint,
					PedSector start, PedSector end);
extern int ped_disk_maximize_partition (PedDisk* disk, PedPartition* part,
					const PedConstraint* constraint);
extern PedGeometry* ped_disk_get_max_partition_geometry (PedDisk* disk,
		PedPartition* part, const PedConstraint* constraint);
extern PedPartition* ped_disk_next_partition (const PedDisk* disk,
					      const PedPartition* part);
*/

%addmethods PedDisk {
  %name(minimize_extended_partition)
     int ped_disk_minimize_extended_partition ();

  %name(get_partition)
     PedPartition* ped_disk_get_partition (int num);
  %name(get_partition_by_sector)
     PedPartition* ped_disk_get_partition_by_sector (PedSector sect);
  %name(extended_partition)
     PedPartition* ped_disk_extended_partition ();
}


%section "filesys"

%rename PedFileSystemType FileSystemType;
typedef struct {
	PedFileSystemType*		next;
  /* FIXME: this should be recognized/declared as constant */
	const char* /*const*/		name;
} PedFileSystemType;

%rename PedFileSystem FileSystem;
typedef struct {
	PedFileSystemType*		type;
	PedGeometry*			geom;
} PedFileSystem;

%addmethods PedFileSystemType {
  %name(register)
     void ped_file_system_type_register ();
  %name(unregister)
     void ped_file_system_type_unregister ();
}

%name(file_system_type_get) PedFileSystemType* ped_file_system_type_get (const char* name);

%addmethods PedFileSystemType {
  %name(get_next)
     PedFileSystemType* ped_file_system_type_get_next ();
}

%addmethods PedGeometry {
  %name(file_system_probe)
     PedFileSystemType* ped_file_system_probe ();
  %name(file_system_clobber)
     int ped_file_system_clobber ();

  %name(file_system_open)
     PedFileSystem* ped_file_system_open ();
  %name(file_system_create)
     PedFileSystem* ped_file_system_create (PedFileSystemType* type);
}

%addmethods PedFileSystem {
  %name(close)
     int ped_file_system_close ();
  %name(check)
     int ped_file_system_check ();
  %name(copy)
     int ped_file_system_copy (PedGeometry* geom);
  %name(resize)
     int ped_file_system_resize (const PedGeometry* geom);
  %name(get_resize_constraint)
     PedConstraint* ped_file_system_get_resize_constraint ();
}


%section "constraint"

%rename PedConstraint Constraint;
typedef struct {
	PedAlignment*	start_align;
	PedAlignment*	end_align;
	PedGeometry*	start_range;
	PedGeometry*	end_range;
	PedSector	min_size;
} PedConstraint;

%addmethods PedConstraint {
  %name(init)
     int ped_constraint_init (const PedAlignment* start_align,
			      const PedAlignment* end_align,
			      const PedGeometry* start_range,
			      const PedGeometry* end_range,
			      PedSector min_size);
}
%name(constraint_new) PedConstraint*
ped_constraint_new (const PedAlignment* start_align,
					const PedAlignment* end_align,
					const PedGeometry* start_range,
					const PedGeometry* end_range,
					PedSector min_size);

%addmethods PedConstraint {
  %name(duplicate)
     PedConstraint* ped_constraint_duplicate ();
  %name(done)
     void ped_constraint_done ();
  %name(destroy)
     void ped_constraint_destroy ();

  %name(intersect)
     PedConstraint* ped_constraint_intersect (const PedConstraint* b);

  %name(solve_max)
     PedGeometry* ped_constraint_solve_max ();

  %name(solve_nearest)
     PedGeometry* ped_constraint_solve_nearest (const PedGeometry* geom);

  %name(is_solution)
     int ped_constraint_is_solution (const PedGeometry* geom);
}

%addmethods PedDisk {
  %name(constraint_any)
     PedConstraint* ped_constraint_any ();
}

%addmethods PedGeometry {
  %name(constraint_exact)
     PedConstraint* ped_constraint_exact ();
}

extern const PedConstraint*	ped_constraint_none;


%section "natmat"

%rename PedAlignment Alignment;
typedef struct {
	PedSector	offset;
	PedSector	grain_size;
} PedAlignment;

%name(round_up_to) PedSector ped_round_up_to (PedSector sector, PedSector grain_size);
%name(round_down_to) PedSector ped_round_down_to (PedSector sector, PedSector grain_size);
%name(round_to_nearest) PedSector ped_round_to_nearest (PedSector sector, PedSector grain_size);
%name(greatest_common_divisor) PedSector ped_greatest_common_divisor (PedSector a, PedSector b);


%name(alignment_new) PedAlignment* ped_alignment_new (PedSector offset, PedSector grain_size);

%addmethods PedAlignment {
  %name(init)
     int ped_alignment_init (PedSector offset,
			     PedSector grain_size);

  %name(destroy)
     void ped_alignment_destroy ();
  %name(duplicate)
     PedAlignment* ped_alignment_duplicate ();
  %name(intersect)
     PedAlignment* ped_alignment_intersect (const PedAlignment* b);

  %name(align_up)
     PedSector ped_alignment_align_up (const PedGeometry* geom,
				       PedSector sector);
  %name(align_down)
     PedSector ped_alignment_align_down (const PedGeometry* geom,
					 PedSector sector);
  %name(align_nearest)
     PedSector ped_alignment_align_nearest (const PedGeometry* geom,
					    PedSector sector);

  %name(is_aligned)
     int ped_alignment_is_aligned (const PedGeometry* geom,
				   PedSector sector);
}

extern const PedAlignment* ped_alignment_any;
extern const PedAlignment* ped_alignment_none;


#ifdef REALLY_USE_EXCEPTIONS
/* not swigified */

%section "exceptions"

enum _PedExceptionType {
	PED_EXCEPTION_INFORMATION=1,
	PED_EXCEPTION_WARNING=2,
	PED_EXCEPTION_ERROR=3,
	PED_EXCEPTION_FATAL=4,
	PED_EXCEPTION_BUG=5,
	PED_EXCEPTION_NO_FEATURE=6,
};
typedef enum _PedExceptionType PedExceptionType;

enum _PedExceptionOption {
	PED_EXCEPTION_UNHANDLED=0,
	PED_EXCEPTION_FIX=1,
	PED_EXCEPTION_YES=2,
	PED_EXCEPTION_NO=4,
	PED_EXCEPTION_OK=8,
	PED_EXCEPTION_RETRY=16,
	PED_EXCEPTION_IGNORE=32,
	PED_EXCEPTION_CANCEL=64,
};
typedef enum _PedExceptionOption PedExceptionOption;
#define PED_EXCEPTION_OK_CANCEL	    (PED_EXCEPTION_OK + PED_EXCEPTION_CANCEL)
#define PED_EXCEPTION_YES_NO	    (PED_EXCEPTION_YES + PED_EXCEPTION_NO)
#define PED_EXCEPTION_YES_NO_CANCEL (PED_EXCEPTION_YES_NO \
						+ PED_EXCEPTION_CANCEL)
#define PED_EXCEPTION_IGNORE_CANCEL (PED_EXCEPTION_IGNORE		\
				     + PED_EXCEPTION_CANCEL)
#define PED_EXCEPTION_RETRY_CANCEL  (PED_EXCEPTION_RETRY + PED_EXCEPTION_CANCEL)
#define PED_EXCEPTION_RETRY_IGNORE_CANCEL (PED_EXCEPTION_RETRY		\
					   + PED_EXCEPTION_IGNORE_CANCEL)
#define PED_EXCEPTION_OPTION_FIRST PED_EXCEPTION_FIX
#define PED_EXCEPTION_OPTION_LAST PED_EXCEPTION_CANCEL

%rename PedException Exception;
typedef struct {
	char*			message;
	PedExceptionType	type;
	PedExceptionOption	options;	/* or'ed list of options that
						   the exception handler can
						   return */
} PedException;

/* FIXME: handler support ? */
/* typedef PedExceptionOption (PedExceptionHandler) (PedException* ex); */

extern int ped_exception;	/* set to true if there's an exception */

extern char* ped_exception_get_type_string (PedExceptionType ex_type);
extern char* ped_exception_get_option_string (PedExceptionOption ex_opt);

extern void ped_exception_set_handler (PedExceptionHandler* handler);

/* FIXME: Variable length arguments not supported
extern PedExceptionOption	ped_exception_throw (PedExceptionType ex_type,
						     PedExceptionOption ex_opt,
						     const char* message,
						     ...);
*/

/* rethrows an exception - i.e. calls the exception handler, (or returns a
   code to return to pass up higher) */
extern PedExceptionOption	ped_exception_rethrow ();

/* frees an exception, indicating that the exception has been handled.
   Calling an exception handler counts. */
extern void			ped_exception_catch ();

/* indicate that exceptions should not go to the exception handler, but passed
   up to the calling function(s) */
extern void			ped_exception_fetch_all ();

/* indicate that exceptions should invoke the exception handler */
extern void			ped_exception_leave_all ();

#endif /* REALLY_USE_EXCEPTIONS */
